@@ -23,11 +23,11 @@
 #include "common_benchmark_header.hpp"
 
 // HIP API
-#include "hipcub/block/block_load.hpp"
-#include "hipcub/block/block_radix_rank.hpp"
-#include "hipcub/block/block_store.hpp"
+#include <hipcub/block/block_load.hpp>
+#include <hipcub/block/block_radix_rank.hpp>
+#include <hipcub/block/block_store.hpp>
 
-#include "hipcub/block/radix_rank_sort_operations.hpp"
+#include <hipcub/block/radix_rank_sort_operations.hpp>
 
 #ifndef DEFAULT_N
 const size_t DEFAULT_N = 1024 * 1024 * 128;
