@@ -18,7 +18,7 @@ left_nav_title = f"hipCUB {version_number} Documentation"
 # for PDF output on Read the Docs
 project = "hipCUB Documentation"
 author = "Advanced Micro Devices, Inc."
-copyright = "Copyright (c) 2024 Advanced Micro Devices, Inc. All rights reserved."
+copyright = "Copyright (c) 2024-2025 Advanced Micro Devices, Inc. All rights reserved."
 version = version_number
 release = version_number
 
@@ -33,3 +33,7 @@ external_projects_current_project = "hipcub"
 
 for sphinx_var in ROCmDocs.SPHINX_VARS:
     globals()[sphinx_var] = getattr(docs_core, sphinx_var)
+
+# Suppresses "WARNING: toctree directive not expected with external-toc"
+# Ideally suppression wouldn't be needed; see sphinx-external-toc#36
+suppress_warnings = ["etoc.toctree"]
