@@ -145,18 +145,18 @@ int main(int argc, char** argv)
     Solve(h_in, h_reference, num_items);
 
     // Allocate problem device arrays
-    int *d_in = NULL;
+    int* d_in = nullptr;
     HIP_CHECK(g_allocator.DeviceAllocate((void**)&d_in, sizeof(int) * num_items));
 
     // Initialize device input
     HIP_CHECK(hipMemcpy(d_in, h_in, sizeof(int) * num_items, hipMemcpyHostToDevice));
 
     // Allocate device output array
-    int *d_out = NULL;
+    int* d_out = nullptr;
     HIP_CHECK(g_allocator.DeviceAllocate((void**)&d_out, sizeof(int) * num_items));
 
     // Allocate temporary storage
-    void            *d_temp_storage = NULL;
+    void*           d_temp_storage     = nullptr;
     size_t          temp_storage_bytes = 0;
     HIP_CHECK(hipcub::DeviceScan::ExclusiveSum(d_temp_storage,
                                                temp_storage_bytes,
