@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2021, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2021-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -36,6 +36,7 @@
 #include "../util_ptx.hpp"
 #include "../util_type.hpp"
 #include "block_scan.hpp"
+
 #include <limits>
 #include <type_traits>
 
@@ -280,7 +281,7 @@ private:
         }
 
         // Ensure run offsets and run values have been writen to shared memory
-        CTA_SYNC();
+        __syncthreads();
     }
 
     template <typename RunLengthT, typename TotalDecodedSizeT>
@@ -300,7 +301,7 @@ private:
         total_decoded_size = static_cast<TotalDecodedSizeT>(decoded_size_aggregate);
 
         // Ensure the prefix scan's temporary storage can be reused (may be superfluous, but depends on scan implementation)
-        CTA_SYNC();
+        __syncthreads();
 
         InitWithRunOffsets(run_values, run_offsets);
     }
