@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2021-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,23 +21,21 @@
 // SOFTWARE.
 
 #include "common_test_header.hpp"
-#include "hipcub/block/block_load.hpp"
-#include "hipcub/block/block_run_length_decode.hpp"
-#include "hipcub/block/block_store.hpp"
+#include <hipcub/block/block_load.hpp>
+#include <hipcub/block/block_run_length_decode.hpp>
+#include <hipcub/block/block_store.hpp>
 
-template<
-    class ItemT,
-    class LengthT,
-    unsigned BlockSize,
-    unsigned RunsPerThread,
-    unsigned DecodedItemsPerThread
->
+template<class ItemT,
+         class LengthT,
+         unsigned BlockSize,
+         unsigned RunsPerThread,
+         unsigned DecodedItemsPerThread>
 struct Params
 {
-    using item_type = ItemT;
-    using length_type = LengthT;
-    static constexpr unsigned block_size = BlockSize;
-    static constexpr unsigned runs_per_thread = RunsPerThread;
+    using item_type                                    = ItemT;
+    using length_type                                  = LengthT;
+    static constexpr unsigned block_size               = BlockSize;
+    static constexpr unsigned runs_per_thread          = RunsPerThread;
     static constexpr unsigned decoded_items_per_thread = DecodedItemsPerThread;
 };
 
@@ -79,46 +77,37 @@ using HipcubBlockRunLengthDecodeTestParams
 
 TYPED_TEST_SUITE(HipcubBlockRunLengthDecodeTest, HipcubBlockRunLengthDecodeTestParams);
 
-template<
-    class ItemT,
-    class LengthT,
-    unsigned BlockSize,
-    unsigned RunsPerThread,
-    unsigned DecodedItemsPerThread
->
+template<class ItemT,
+         class LengthT,
+         unsigned BlockSize,
+         unsigned RunsPerThread,
+         unsigned DecodedItemsPerThread>
 __global__
 __launch_bounds__(BlockSize)
-void block_run_length_decode_kernel(
-    const ItemT * d_run_items,
-    const LengthT * d_run_lengths,
-    ItemT * d_decoded_items)
+void block_run_length_decode_kernel(const ItemT*   d_run_items,
+                                    const LengthT* d_run_lengths,
+                                    ItemT*         d_decoded_items)
 {
-    using BlockRunLengthDecodeT = hipcub::BlockRunLengthDecode<
-        ItemT,
-        BlockSize,
-        RunsPerThread,
-        DecodedItemsPerThread
-    >;
+    using BlockRunLengthDecodeT
+        = hipcub::BlockRunLengthDecode<ItemT, BlockSize, RunsPerThread, DecodedItemsPerThread>;
     static constexpr unsigned int decoded_items_per_block = BlockSize * DecodedItemsPerThread;
     __shared__ typename BlockRunLengthDecodeT::TempStorage temp_storage;
 
-    ItemT run_items[RunsPerThread];
+    ItemT   run_items[RunsPerThread];
     LengthT run_lengths[RunsPerThread];
 
     const unsigned global_thread_idx = BlockSize * hipBlockIdx_x + hipThreadIdx_x;
     hipcub::LoadDirectBlocked(global_thread_idx, d_run_items, run_items);
     hipcub::LoadDirectBlocked(global_thread_idx, d_run_lengths, run_lengths);
 
-    unsigned total_decoded_size{};
-    BlockRunLengthDecodeT block_run_length_decode(
-        temp_storage,
-        run_items,
-        run_lengths,
-        total_decoded_size
-    );
+    unsigned              total_decoded_size{};
+    BlockRunLengthDecodeT block_run_length_decode(temp_storage,
+                                                  run_items,
+                                                  run_lengths,
+                                                  total_decoded_size);
 
     unsigned decoded_window_offset = 0;
-    while (decoded_window_offset < total_decoded_size)
+    while(decoded_window_offset < total_decoded_size)
     {
         ItemT decoded_items[DecodedItemsPerThread];
 
@@ -139,13 +128,13 @@ TYPED_TEST(HipcubBlockRunLengthDecodeTest, TestDecode)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using ItemT = typename TestFixture::params::item_type;
-    using LengthT = typename TestFixture::params::length_type;
-    constexpr unsigned block_size = TestFixture::params::block_size;
-    constexpr unsigned runs_per_thread = TestFixture::params::runs_per_thread;
+    using ItemT                                 = typename TestFixture::params::item_type;
+    using LengthT                               = typename TestFixture::params::length_type;
+    constexpr unsigned block_size               = TestFixture::params::block_size;
+    constexpr unsigned runs_per_thread          = TestFixture::params::runs_per_thread;
     constexpr unsigned decoded_items_per_thread = TestFixture::params::decoded_items_per_thread;
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
         const unsigned int seed_value
             = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
@@ -154,93 +143,83 @@ TYPED_TEST(HipcubBlockRunLengthDecodeTest, TestDecode)
         const LengthT max_run_length = static_cast<LengthT>(
             std::min(1000ll, static_cast<long long>(std::numeric_limits<LengthT>::max())));
 
-        size_t num_runs = runs_per_thread * block_size;
-        auto run_items = test_utils::get_random_data<ItemT>(
-            num_runs,
-            std::numeric_limits<ItemT>::min(),
-            std::numeric_limits<ItemT>::max(),
-            seed_value
-        );
-        auto run_lengths = test_utils::get_random_data<LengthT>(
-            num_runs,
-            static_cast<LengthT>(1),
-            max_run_length,
-            seed_value
-        );
+        size_t num_runs    = runs_per_thread * block_size;
+        auto   run_items   = test_utils::get_random_data<ItemT>(num_runs,
+                                                            std::numeric_limits<ItemT>::min(),
+                                                            std::numeric_limits<ItemT>::max(),
+                                                            seed_value);
+        auto   run_lengths = test_utils::get_random_data<LengthT>(num_runs,
+                                                                static_cast<LengthT>(1),
+                                                                max_run_length,
+                                                                seed_value);
 
-        std::default_random_engine prng(seed_value);
+        std::default_random_engine            prng(seed_value);
         std::uniform_int_distribution<size_t> num_empty_runs_dist(1, 4);
-        const size_t num_trailing_empty_runs = num_empty_runs_dist(prng);
+        const size_t                          num_trailing_empty_runs = num_empty_runs_dist(prng);
         num_runs += num_trailing_empty_runs;
 
-        const auto empty_run_items = test_utils::get_random_data<ItemT>(
-            num_trailing_empty_runs,
-            std::numeric_limits<ItemT>::min(),
-            std::numeric_limits<ItemT>::max(),
-            seed_value
-        );
+        const auto empty_run_items
+            = test_utils::get_random_data<ItemT>(num_trailing_empty_runs,
+                                                 std::numeric_limits<ItemT>::min(),
+                                                 std::numeric_limits<ItemT>::max(),
+                                                 seed_value);
+        // Not strictly required, but fixes a spurious GCC warning and good practice anyways
+        run_items.reserve(run_items.size() + empty_run_items.size());
         run_items.insert(run_items.end(), empty_run_items.begin(), empty_run_items.end());
+        run_lengths.reserve(run_lengths.size() + num_trailing_empty_runs);
         run_lengths.insert(run_lengths.end(), num_trailing_empty_runs, static_cast<LengthT>(0));
 
         std::vector<ItemT> expected;
-        for (size_t i = 0; i < run_items.size(); ++i)
+        for(size_t i = 0; i < run_items.size(); ++i)
         {
-            for (size_t j = 0; j < static_cast<size_t>(run_lengths[i]); ++j)
+            for(size_t j = 0; j < static_cast<size_t>(run_lengths[i]); ++j)
             {
                 expected.push_back(run_items[i]);
             }
         }
 
-        ItemT * d_run_items{};
-        HIP_CHECK(test_common_utils::hipMallocHelper(&d_run_items, run_items.size() * sizeof(ItemT)));
+        ItemT* d_run_items{};
         HIP_CHECK(
-            hipMemcpy(
-                d_run_items,
-                run_items.data(),
-                run_items.size() * sizeof(ItemT),
-                hipMemcpyHostToDevice
-            )
-        );
+            test_common_utils::hipMallocHelper(&d_run_items, run_items.size() * sizeof(ItemT)));
+        HIP_CHECK(hipMemcpy(d_run_items,
+                            run_items.data(),
+                            run_items.size() * sizeof(ItemT),
+                            hipMemcpyHostToDevice));
 
-        LengthT * d_run_lengths{};
-        HIP_CHECK(test_common_utils::hipMallocHelper(&d_run_lengths, run_lengths.size() * sizeof(LengthT)));
+        LengthT* d_run_lengths{};
+        HIP_CHECK(test_common_utils::hipMallocHelper(&d_run_lengths,
+                                                     run_lengths.size() * sizeof(LengthT)));
+        HIP_CHECK(hipMemcpy(d_run_lengths,
+                            run_lengths.data(),
+                            run_lengths.size() * sizeof(LengthT),
+                            hipMemcpyHostToDevice));
+
+        ItemT* d_decoded_runs{};
         HIP_CHECK(
-            hipMemcpy(
-                d_run_lengths,
-                run_lengths.data(),
-                run_lengths.size() * sizeof(LengthT),
-                hipMemcpyHostToDevice
-            )
-        );
-
-        ItemT * d_decoded_runs{};
-        HIP_CHECK(test_common_utils::hipMallocHelper(&d_decoded_runs, expected.size() * sizeof(ItemT)));
+            test_common_utils::hipMallocHelper(&d_decoded_runs, expected.size() * sizeof(ItemT)));
 
+        HIP_CHECK(hipGetLastError());
         hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(
-                block_run_length_decode_kernel<
-                    ItemT,
-                    LengthT,
-                    block_size,
-                    runs_per_thread,
-                    decoded_items_per_thread
-                >
-            ),
-            dim3(1), dim3(block_size), 0, 0,
-            d_run_items, d_run_lengths, d_decoded_runs
-        );
-        HIP_CHECK(hipPeekAtLastError());
+            HIP_KERNEL_NAME(block_run_length_decode_kernel<ItemT,
+                                                           LengthT,
+                                                           block_size,
+                                                           runs_per_thread,
+                                                           decoded_items_per_thread>),
+            dim3(1),
+            dim3(block_size),
+            0,
+            0,
+            d_run_items,
+            d_run_lengths,
+            d_decoded_runs);
+        HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         std::vector<ItemT> output(expected.size());
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(),
-                d_decoded_runs,
-                output.size() * sizeof(ItemT),
-                hipMemcpyDeviceToHost
-            )
-        );
+        HIP_CHECK(hipMemcpy(output.data(),
+                            d_decoded_runs,
+                            output.size() * sizeof(ItemT),
+                            hipMemcpyDeviceToHost));
 
         HIP_CHECK(hipFree(d_run_items));
         HIP_CHECK(hipFree(d_run_lengths));
